import 'dart:convert';

import 'package:dio/dio.dart';
import 'package:flutter_mylibrary/utils/http/response_bean.dart';
import 'package:flutter_mylibrary/utils/utils.dart';

class DioUtils {

  //服务器路径
  static final _baseUrl = '';

  static DioUtils instance;

  BaseOptions options;

  ResponseBean responseBean;

  Dio dio;

  bool _isDeBug = false;

  String _defSuccessCode = '200';

  static DioUtils getInstance() {
    if(instance == null){
      // 保持本地实例直到完全初始化。
      return DioUtils();
    }else{
      return instance;
    }
  }

  ///-----------------------------------------工具类初始化设置-------------------------------------
  DioUtils(){
    //基础配置
    options = new BaseOptions(
        baseUrl: _baseUrl,
        connectTimeout: 5000,
        receiveTimeout: 5000,
    );
    // 初始化
    dio = new Dio(options);

    //请求成功拦截，全局处理请求
    dio.interceptors.add(InterceptorsWrapper(
        onRequest: (RequestOptions requestOptions){
          //开始请求
          responseBean = ResponseBean();
          return requestOptions; //continue
        },
        onResponse:(Response response) {
          _printHttpLog(response);
          //可以在这里全局处理请求
          Map map = json.decode(response.toString());
          responseBean = ResponseBean.fromJson(map);
          responseBean.response = response;
          //装填responseBean
          if(responseBean.code != _defSuccessCode){
            throw Exception('错误代码: ' + responseBean.code + '----错误信息: ' + responseBean.message);
          }
          return responseBean.response;
        },
        onError: (DioError e) {
          Utils.showToast(responseBean.message);
        }
    ));
  }


  ///-----------------------------------------请求方法封装-------------------------------------
  /// post 请求封装
  Future<ResponseBean> post(url, {params}) async {
    await dio.post(
      url,
      data:params,
    );
    return responseBean;
  }



  ///-----------------------------------------dio部分设置-------------------------------------
  void openDebug(){
    _isDeBug = true;
  }

  void closeDebug(){
    _isDeBug = false;
  }


  ///用于添加拦截器,可以设置全局请求处理
  void setinterceptors(InterceptorsWrapper interceptorsWrapper){
    dio.interceptors.clear();
    dio.interceptors.add(interceptorsWrapper);
  }


  void setOptions(BaseOptions baseOptions){
    dio.options = baseOptions;
  }


  void setReqsponseSuccessCode(String code){
    _defSuccessCode = code;
  }



  ///-----------------------------------------debug打印方法-------------------------------------
  /// print Http Log.
  void _printHttpLog(Response response) {
    if (!_isDeBug) {
      return;
    }
    try {
      print("----------------Http Log----------------" +
          "\n[statusCode]:   " +
          response.statusCode.toString() +
          "\n[request   ]:   " +
          _getOptionsStr(response.request));
      _printDataStr("reqdata ", response.request.data);
      _printDataStr("response", response.data);
      print("----------------end----------------");
    } catch (ex) {
      print("Http Log" + " error......");
    }
  }


  /// get Options Str.
  String _getOptionsStr(RequestOptions request) {
    return "method: " +
        request.method +
        "  baseUrl: " +
        request.baseUrl +
        "  path: " +
        request.path;
  }


  /// print Data Str.
  void _printDataStr(String tag, Object value) {
    String da = value.toString();
    while (da.isNotEmpty) {
      if (da.length > 512) {
        print("[$tag  ]:   " + da.substring(0, 512));
        da = da.substring(512, da.length);
      } else {
        print("[$tag  ]:   " + da);
        da = "";
      }
    }
  }
}